SuperStrict

Framework BRL.LinkedList
Import BRL.Max2D
Import BRL.Timer
Import BRL.Random
Import BRL.Graphics
Import BRL.GLMax2D
Import BRL.PNGLoader
Import BRL.Map

'-----------------------

Type TProjectile
	Global gList:TList = New TList
	
	Field mImage:TImage
	Field mSpeedX:Float,mSpeedY:Float
	Field mDamage:Int = 10
	Field mShotBy:TVessel
	
	Field mX:Float,mY:Float
	
	Method New()
		gList.AddLast(Self)
	End Method
End Type

Type TExplosion
	Global gList:TList = New TList
	
	Field mX:Float,mY:Float
	Field mFinTime:Int
	Field mImageID:Int
	
	Method New()
		gList.AddLast(Self)
		mFinTime = MilliSecs() + 2000
	End Method
End Type

'---------

Type TVesselBlueprint
	Global gMap:TMap = New TMap
	
	Field mFilename:String
	Field mImage:TImage
	Field mSpeedX:Float,mSpeedY:Float
	Field mHealth:Int
	Field mCrashDamage:Int
	Field mWeaponID:Int
	Field mScore:Int
	
	Function get:TVesselBlueprint(filename:String)
		Return TVesselBlueprint(gMap.ValueForKey(filename))
	End Function
End Type

'---------

Type TVessel
	Global gList:TList = New TList
	
	Field mBlueprint:TVesselBlueprint
	
	Field mX:Float,mY:Float
	Field mHealth:Int = 0
	Field mLastShot:Int = MilliSecs()
	
	Field mLastUpdate:Int = MilliSecs()
	
	Method New()
		gList.AddLast(Self)
	End Method
	
	Method kill()
		Self.mHealth = 0
		
		Local explosion:TExplosion = New TExplosion
		explosion.mX = Self.mX
		explosion.mY = Self.mY
		
		TVessel.gList.Remove(Self)
	End Method
End Type

'---------

Type TPatternEntry
	Field mBlueprint:TVesselBlueprint
	Field mX:Float,mY:Float
End Type

Type TPattern
	Global gList:TList = New TList
	
	Method New()
		gList.AddLast(Self)
	End Method
	
	Field mScoreExecMin:Int
	Field mScoreExecMax:Int
	
	Field mExecIntervall:Int
	Field mExecIntervallVary:Float
	
	Field mNextExec:Int
	
	Field mPatternList:TList = New TList
	
	Method exec()
		For Local patternentry:TPatternEntry = EachIn mPatternList
			Local newVessel:TVessel = New TVessel
				newVessel.mBlueprint = patternentry.mBlueprint
				newVessel.mHealth = newVessel.mBlueprint.mHealth
				
				newVessel.mX = patternentry.mX
				newVessel.mY = patternentry.mY
				
				newVessel.mLastShot = MilliSecs() + gWeaponCooldowns[newVessel.mBluePrint.mWeaponID]*Rnd(5.0,10.0)
		Next
		
		Self.mNextExec = MilliSecs() + Self.mExecIntervall * Rnd(1.0,Self.mExecIntervallVary)
	End Method
End Type

'-----------------------

Type TBonus
	Global gList:TList = New TList
	
	Field mX:Float,mY:Float
	
	Method New()
		gList.AddLast(Self)
		
		mX = 25 + Rnd(0.0,1.0)*(800-50)
		mY = -50
	End Method
End Type

'-----------------------

Global gFullscreen:Byte
Global gRefreshRate:Int
AutoMidHandle(True)

parseSettings()
parseBlueprints()
parsePatterns()

loadHighscore()

'-----------------------

Graphics 800,600,32*gFullscreen
AppTitle = "Space Strike 2 - BCC #41 - Kernle - v1.01"

Global gFPSTimer:TTimer = CreateTimer(gRefreshRate)

'-----------------------

Global gPlayerImage:TImage = LoadAnimImage("./Textures/player.png",97,66,0,13)
Global gHealthImage:TImage = LoadAnimImage("./Textures/health.png",50,50,0,9)

'-----------------------

Global gTitleImage:TImage = LoadImage("./Textures/title.png")
Global gInfoImage:TImage = LoadImage("./Textures/info.png")

Global gGamveOverImage:TImage = LoadImage("./Textures/gameover.png")
Global gWarningImage:TImage = LoadImage("./Textures/warning.png")

Global gNumericImage:TImage[10]
For Local I:Int = 0 To 9
	gNumericImage[I] = LoadImage("./Textures/Num/"+I+".png")
Next

Global gExplosionImage:TImage[2]
For Local I:Int = 0 To 1
	gExplosionImage[I] = LoadImage("./Textures/explosion"+(I+1)+".png")
Next

'-----------------------

Global gBGImage:TImage = LoadImage("./Textures/stars.png")
Global gBGImage2:TImage = LoadImage("./Textures/cloud.png")

Global gProjectileImage:TImage[3]
	gProjectileImage[0] = LoadImage("./Textures/laser.png")
	gProjectileImage[1] = LoadImage("./Textures/laser2.png")
	gProjectileImage[2] = LoadImage("./Textures/laser3.png")
	
Global gWeaponCooldowns:Int[3]
	gWeaponCooldowns[0] = 100
	gWeaponCooldowns[1] = 100
	gWeaponCooldowns[2] = 100
	
Const BOUNDSAFE:Int = 200

Global gInScreenImage:TImage = CreateImage(800+BOUNDSAFE,600+BOUNDSAFE*2)
	Local map:TPixmap = LockImage(gInScreenImage)
	For Local x:Int = 0 To gInScreenImage.Width-1
		For Local y:Int = 0 To gInScreenImage.Height-1
			WritePixel(map,x,y,$ff000000)
		Next
	Next
	UnlockImage(gInScreenImage)

SetImageHandle(gInScreenImage,BOUNDSAFE,BOUNDSAFE*2)

'-----------------------

HideMouse()
SetBlend(ALPHABLEND)

'-----------------------

Global gGameState:Byte = GAMESTATEMENUE
	Const GAMESTATEMENUE:Byte = 1
	Const GAMESTATEPREGAME:Byte = 2
	Const GAMESTATEGAME:Byte = 3

'-----------------------

Const BONUSCASTINTERVAL:Int = 5000
Const BONUSCASTMODIFIERMIN:Float = 0.7
Const BONUSCASTMODIFIERMAX:Float = 3.0

Global gNextBonusCast:Int

'-----------------------

Const PLAYERMAXHEALTH:Int = 1000

Global mPlayerFrame:Int
Global gPlayerHealth:Int
Global gPlayerWeaponBar:Float
	Const WEAPONBARMAX:Float = 3000
	Const WEAPONBARINCR:Float = 1200
	Const WEAPONBARDECR:Float = 0.1

Global gPlayerScore:Int
Global gPlayerMaxScore:Int
Global gPlayerLastShot:Int
Global gPlayerX:Float
Global gPlayerY:Float
Global gGameOver:Byte 

prepareGameEnvi()

'-----------------------

Global gLastTime:Int = MilliSecs()
Global gSpeedMod:Float = 1.0

'-----------------------

Repeat
	Cls
	
	'-----------------------
	
	Select gGameState
		Case GAMESTATEMENUE
			proccessMenue()
		Case GAMESTATEPREGAME
			proccessPreGame()
		Case GAMESTATEGAME
			proccessGame()
	End Select
	
	'-----------------------
	
	gSpeedMod = MilliSecs() - gLastTime
	gLastTime = MilliSecs()
	
	Flip 0
	If gFPSTimer Then WaitTimer(gFPSTimer)
Until AppTerminate()

'-----------------------

Global gTileShift:Float = 0
Global gTileSpeed:Float = 0.125

Global gTileShift2:Float = 0
Global gTileSpeed2:Float = 0.20

Function proccessMenue()
	'-----------------------
	' Rendering Background
	'-----------------------
	
	renderBackground()
	
	'-----------------------
	' Rendering player
	'-----------------------
	
	DrawImage(gPlayerImage,600,360,6)
	
	If MouseHit(1) Then gGameState = GAMESTATEPREGAME ; prepareGameEnvi()
	If KeyHit(KEY_ESCAPE) Then End
	
	'-----------------------
	' Rendering Foreground
	'-----------------------
	
	renderForeground()
	
	'-----------------------
	' Rendering Overlay
	'-----------------------
	
	DrawImage(gTitleImage,400,300)
	renderNumeric(gPlayerMaxScore,207,575)
End Function

Function proccessPreGame()
	'-----------------------
	' Rendering Background
	'-----------------------
	
	renderBackground()
	
	'-----------------------
	' Rendering player
	'-----------------------
	
	DrawImage(gPlayerImage,gPlayerX,gPlayerY,6)
	
	If MouseHit(1) Then gGameState = GAMESTATEGAME ; MoveMouse(gPlayerX,gPlayerY) ; prepareGameEnvi()
	If KeyHit(KEY_ESCAPE) Then gGameState = GAMESTATEMENUE
	
	'-----------------------
	' Rendering Foreground
	'-----------------------
	
	renderForeground()
	
	'-----------------------
	' Texts and debug stuff
	'-----------------------
	
	renderHealthbar()
	renderWeaponbar()
	
	renderNumeric(0,GraphicsWidth() - calcNumericLength(0),20)
	
	'-----------------------
	' Rendering Overlay
	'-----------------------
	
	DrawImage(gInfoImage,400,300)
End Function

Function proccessGame()
	'-----------------------
	' Movement logic
	'-----------------------
	
	If MouseX() > 800-50 Then MoveMouse(800-50,MouseY())
	If MouseX() < 50 Then MoveMouse(50,MouseY())
	
	If MouseY() > 600-33 Then MoveMouse(MouseX(),600-33)
	If MouseY() < 33 Then MoveMouse(MouseX(),33)
	
	
	Local xSpeed:Float = (gPlayerX-MouseX())*0.05
	Local ySpeed:Float = (gPlayerY-MouseY())*0.05
	
	If(Abs(xSpeed) > 10) Then xSpeed = 10*Sgn(xSpeed)
	If(Abs(ySpeed) > 10) Then ySpeed = 10*Sgn(ySpeed)
	
	gPlayerX:-xSpeed*gSpeedMod/10
	gPlayerY:-YSpeed*gSpeedMod/10
	
	If(Abs(xSpeed) < 1) Then xSpeed = 0
	
	If KeyHit(KEY_ESCAPE) Then gGameState = GAMESTATEMENUE ; FlushMouse()
	
	'-----------------------
	' Rendering Background
	'-----------------------
	
	renderBackground()
	
	'-----------------------
	' Rendering player (dependent on xSpeed -> degree frame)
	'-----------------------
	
	If gGameOver = False
		mPlayerFrame = 6 - xSpeed/10*5
		DrawImage(gPlayerImage,gPlayerX,gPlayerY,mPlayerFrame)
		
		gPlayerWeaponBar:-WEAPONBARDECR*gSpeedMod
		If gPlayerWeaponBar < 0 Then gPlayerWeaponBar = 0
		
		Local currentweapon:Int = gPlayerWeaponBar / WEAPONBARMAX * 2
		
		If (MouseHit(1) Or MouseHit(2)) And gPlayerLastShot + gWeaponCooldowns[currentweapon] < MilliSecs()
			shootProjectile()
			gPlayerLastShot = MilliSecs()
		EndIf
		
		'-----------------------
		' Rendering movement line
		'-----------------------
		
		SetAlpha(0.25)
		DrawLine(gPlayerX,gPlayerY,MouseX(),MouseY())
		DrawImage(gPlayerImage,MouseX(),MouseY(),mPlayerFrame)
		SetAlpha(1.0)
	EndIf
	
	'-----------------------
	' Updating and rendering bonus items
	'-----------------------
	
	If gNextBonusCast < MilliSecs()
		Local bonus:TBonus = New TBonus
		gNextBonusCast = MilliSecs()+BONUSCASTINTERVAL*Rnd(BONUSCASTMODIFIERMIN,BONUSCASTMODIFIERMAX)
	EndIf
	
	updateBonus()
	
	'-----------------------
	' Updating and rendering vessels
	'-----------------------
	
	For Local pattern:TPattern = EachIn TPattern.gList
		If gPlayerScore >= pattern.mScoreExecMin And (gPlayerScore <= pattern.mScoreExecMax Or pattern.mScoreExecMax = -1)
			'not executed yet
			If pattern.mNextExec = 0 Then pattern.mNextExec = MilliSecs() + pattern.mExecIntervall * Rnd(1.0,pattern.mExecIntervallVary)
			
			'it's time to execute
			If pattern.mNextExec < MilliSecs() Then pattern.exec()
		EndIf
	Next
	
	updateVessels()
	
	'-----------------------
	' Updating and rendering projectiles
	'-----------------------
	
	updateProjectiles()
	updateExplosions()
	
	'-----------------------
	' Rendering Foreground
	'-----------------------
	
	renderForeground()
	
	'-----------------------
	' Texts and debug stuff
	'-----------------------
	
	If gGameOver = False
		renderHealthbar()
		renderWeaponbar()
	Else
		DrawImage(gGamveOverImage,GraphicsWidth()/2,GraphicsHeight()/2)
	EndIf
	
	renderNumeric(gPlayerScore,GraphicsWidth() - calcNumericLength(gPlayerScore),20)
End Function

Function renderBackground()
	SetAlpha(1.0)
	TileImage(gBGImage,0,gTileShift)
	gTileShift:+gTileSpeed*gSpeedMod
End Function

Function renderForeground()
	SetBlend(LIGHTBLEND)
	TileImage(gBGImage2,0,gTileShift2)
	gTileShift2:+gTileSpeed2*gSpeedMod
	SetBlend(ALPHABLEND)
End Function

Function renderHealthbar()
	Local health:Float = Float(gPlayerHealth) / Float(PLAYERMAXHEALTH)
	
	SetAlpha(0.7)
	DrawRect(0,0,200,16)
	
	If(health <= 0.4) Then SetColor(255,0,0) ; SetAlpha(Abs(Sin(MilliSecs()/5)))
	If(health > 0.4 And health <= 0.6) Then SetColor(255,255,0)
	If(health > 0.6 And health <= 1.0) Then SetColor(0,255,0)
	
	DrawRect(2,2,health*(200-4),16-4)
	SetColor(255,255,255)
	
	If health <= 0.4 Then DrawImage(gWarningImage,GraphicsWidth()/2,GraphicsHeight()/2)
	SetAlpha(1.0)
End Function

Function renderWeaponbar()
	Local barlevel:Float = gPlayerWeaponBar / WEAPONBARMAX
	
	SetAlpha(0.7)
	DrawRect(0,20,200,16)
	
	SetAlpha(0.5 + Abs(Sin(MilliSecs()/5))/2)
	SetColor(100,100,255)
	DrawRect(2,22,barlevel*(200-4),16-4)
	
	SetColor(0,0,0)
	SetAlpha(1.0)
	DrawRect(2 + (1.0/3.0)*200-4,22,1,16-4)
	DrawRect(2 + (2.0/3.0)*200-4,22,1,16-4)
	
	SetColor(255,255,255)
End Function

Function renderNumeric(num:String,x:Int,y:Int)
	For Local I:Int = 0 To Len(num)-1
		Local curNum:Int = Chr(num[I]).toInt()
		DrawImage(gNumericImage[curNum],x,y)
		x:+gNumericImage[curNum].width
	Next
End Function

Function calcNumericLength:Int(num:String)
	Local x:Int
	For Local I:Int = 0 To Len(num)-1
		x:+gNumericImage[Chr(num[I]).toInt()].width
	Next
	Return x
End Function

Function updateProjectiles()
	SetColor(255,255,255)
	SetBlend(LIGHTBLEND)
	For Local projectile:TProjectile = EachIn TProjectile.gList
		projectile.mX:+projectile.mSpeedX*gSpeedMod
		projectile.mY:+projectile.mSpeedY*gSpeedMod
		
		Local projImage:TImage = projectile.mImage
		
		DrawImage(projImage,projectile.mX,projectile.mY)
		
		If projectile.mShotBy And gGameOver = False And ImagesCollide(projImage,projectile.mX,projectile.mY,0,gPlayerImage,gPlayerX,gPlayerY,mPlayerFrame)
			'Collision check projectile - player
			gPlayerHealth:-projectile.mDamage
			If gPlayerHealth <= 0 Then playerDeath()
			
			TProjectile.gList.Remove(projectile)
		ElseIf Not ImagesCollide(projImage,projectile.mX,projectile.mY,0,gInScreenImage,0,0,0)
			'Collision check projectile - outside game
			TProjectile.gList.Remove(projectile)
		ElseIf gGameOver = False And Not projectile.mShotBy 'Shot by player
			'Collision check projectile - enemy
			For Local vessel:TVessel = EachIn TVessel.gList
				If ImagesCollide(projImage,projectile.mX,projectile.mY,0,vessel.mBlueprint.mImage,vessel.mX,vessel.mY,0)
					vessel.mHealth:-projectile.mDamage
					If vessel.mHealth <= 0 Then vessel.kill() ; gPlayerScore:+vessel.mBlueprint.mScore ; gPlayerWeaponBar:+200
					If gPlayerWeaponBar > WEAPONBARMAX Then gPlayerWeaponBar = WEAPONBARMAX
					
					TProjectile.gList.Remove(projectile)
					Exit
				EndIf
			Next
		EndIf
	Next
	SetBlend(ALPHABLEND)
End Function

Function updateVessels()
	For Local vessel:TVessel = EachIn TVessel.gList
		Local projImage:TImage = vessel.mBlueprint.mImage
		
		vessel.mY:+ gTileSpeed*gSpeedMod
		
		vessel.mX:+vessel.mBlueprint.mSpeedX*gSpeedMod
		vessel.mY:+vessel.mBlueprint.mSpeedY*gSpeedMod
		
		Local destruction:Int = Float(vessel.mHealth)/Float(vessel.mBlueprint.mHealth)*255
		
		SetColor(255,destruction,destruction)
		DrawImage(projImage,vessel.mX,vessel.mY)
		
		If vessel.mLastShot < MilliSecs()
			shootProjectile(vessel)
			vessel.mLastShot = MilliSecs() + gWeaponCooldowns[vessel.mBluePrint.mWeaponID]*Rnd(10.0,20.0)
		EndIf
		
		If gGameOver = False And ImagesCollide(projImage,vessel.mX,vessel.mY,0,gPlayerImage,gPlayerX,gPlayerY,mPlayerFrame)
			'Collision check enemy - player
			gPlayerHealth:-vessel.mBlueprint.mCrashDamage
			If gPlayerHealth <= 0 Then playerDeath()
			
			vessel.kill()
		ElseIf Not ImagesCollide(projImage,vessel.mX,vessel.mY,0,gInScreenImage,0,0,0)
			'Collision check enemy - outside game
			TVessel.gList.Remove(vessel)
		EndIf
	Next
End Function

Function updateExplosions()
	For Local explosion:TExplosion = EachIn TExplosion.gList
		If explosion.mFinTime < MilliSecs() Then TExplosion.gList.Remove(explosion) ; Continue
		
		explosion.mY:+ gTileSpeed*gSpeedMod
		
		Local alpha:Float = 1 - Float(explosion.mFinTime - MilliSecs()) / 2000.0
		
		SetAlpha(1-alpha)
		
		Select(explosion.mImageID)
			Case 0
				alpha = 1+alpha
			Case 1
				alpha = 1+alpha*3
		End Select
		
		SetScale(alpha,alpha)
		
		DrawImage(gExplosionImage[explosion.mImageID],explosion.mX,explosion.mY)
	Next
	
	SetScale(1,1)
	SetAlpha(1.0)
End Function

Function updateBonus()
	For Local bonus:TBonus = EachIn TBonus.gList		
		bonus.mY:+ gTileSpeed*gSpeedMod
		
		DrawImage(gHealthImage,bonus.mX,bonus.mY,(MilliSecs()/100) Mod 9)
		
		If gGameOver = False And ImagesCollide(gHealthImage,bonus.mX,bonus.mY,0,gPlayerImage,gPlayerX,gPlayerY,mPlayerFrame)
			'Collision check bonus - player
			gPlayerHealth:+100
			If gPlayerHealth > PLAYERMAXHEALTH Then gPlayerHealth = PLAYERMAXHEALTH
			
			gPlayerScore:+1000
			
			TBonus.gList.Remove(bonus)
		ElseIf Not ImagesCollide(gHealthImage,bonus.mX,bonus.mY,0,gInScreenImage,0,0,0)
			'Collision check bonus - outside game
			TBonus.gList.Remove(bonus)
		EndIf
	Next
End Function

Function playerDeath()
	gPlayerHealth = 0
	gGameOver = True
	saveHighscore()
	
	Local explosion:TExplosion = New TExplosion
		explosion.mX = gPlayerX
		explosion.mY = gPlayerY
		explosion.mImageID = 1
End Function

Function shootProjectile(vessel:TVessel = Null)
	Global internFireCount:Int = 0
	
	Local weaponID:Int = gPlayerWeaponBar / WEAPONBARMAX * 3
	Local direction:Int = 1
	
	If vessel
		direction = -1 'down
		weaponID = vessel.mBluePrint.mWeaponID
	EndIf
	
	Local projectile:TProjectile = New TProjectile
		projectile.mImage = gProjectileImage[weaponID]
		projectile.mShotBy = vessel
	
	Select weaponID
		Case 0
			projectile.mSpeedX = 0.0 * direction
			projectile.mSpeedY = -0.7 * direction
			projectile.mDamage = 30
		Case 1
			projectile.mSpeedX = 0.0 * direction
			projectile.mSpeedY = -0.8 * direction
			projectile.mDamage = 20
			
			internFireCount:+1
			
			Select internFireCount
				Case 1
					projectile.mX = 5
				Case 2
					projectile.mX = -5
					internFireCount = 0
			End Select
			
			If internFireCount <> 0 Then shootProjectile(vessel)
		Case 2
			projectile.mSpeedX = 0.0 * direction
			projectile.mSpeedY = -0.9 * direction
			projectile.mDamage = 20
			
			internFireCount:+1
			
			Select internFireCount
				Case 1
					projectile.mX = 40
					projectile.mY = 50
				Case 2
					projectile.mX = -40
					projectile.mY = 50
				Case 3
					projectile.mX = 5
				Case 4
					projectile.mX = -5
					internFireCount = 0
			End Select
			
			If internFireCount <> 0 Then shootProjectile(vessel)
	End Select
	
	If vessel
		projectile.mX:+vessel.mX + Rnd(-5,5)
		projectile.mY:+vessel.mY + ImageHeight(vessel.mBlueprint.mImage)/2 + ImageHeight(projectile.mImage)/2
	Else
		projectile.mX:+gPlayerX
		projectile.mY:+gPlayerY - ImageHeight(gPlayerImage)/2 - ImageHeight(projectile.mImage)/2
	EndIf
End Function

Function parseSettings()
	If FileType("settings.ini") = 0
		Local filestream:TStream = WriteFile("settings.ini")
		If(filestream)
			WriteLine(filestream,"60")
			WriteLine(filestream,"False")
			CloseStream(filestream)
		EndIf
		
		gRefreshRate = 60
		gFullscreen = False
	Else
		Local filestream:TStream = ReadFile("settings.ini")
		
		If(filestream)
			gRefreshRate = ReadLine(filestream).toInt()
			Local fullscreen:String = ReadLine(filestream).toLower()
			
			If fullscreen = "true" Or fullscreen.toInt() > 0 Then gFullscreen = True
			CloseStream(filestream)
		Else
			gRefreshRate = 60
			gFullscreen = False
		EndIf
	EndIf
End Function

Function parseBlueprints()
	Local dirstring:String = "./Blueprints/"
	Local dir:String[] = LoadDir(dirstring)
	If Not dir RuntimeError "failed to read blueprints"
	
	For Local file:String = EachIn dir
		Local filestream:TStream = ReadFile(dirstring + file)
		
		Local instance:TVesselBlueprint = New TVesselBlueprint
		
		instance.mFilename = StripExt(file)
		instance.mImage = LoadImage(ReadLine(filestream))
		instance.mSpeedX = ReadLine(filestream).toFloat()
		instance.mSpeedY = ReadLine(filestream).toFloat()
		instance.mHealth = ReadLine(filestream).toInt()
		instance.mCrashDamage = ReadLine(filestream).toInt()
		instance.mWeaponID = ReadLine(filestream).toInt()
		instance.mScore = ReadLine(filestream).toInt()
		
		TVesselBlueprint.gMap.Insert(instance.mFilename,instance)
	Next
End Function


Function parsePatterns()
	Local dirstring:String = "./Patterns/"
	Local dir:String[] = LoadDir(dirstring)
	If Not dir RuntimeError "failed to read patterns"
	
	For Local file:String = EachIn dir
		Local filestream:TStream = ReadFile(dirstring + file)
		
		Local instance:TPattern = New TPattern
			instance.mScoreExecMin = ReadLine(filestream).toInt()
			instance.mScoreExecMax = ReadLine(filestream).toInt()
			
			instance.mExecIntervall = ReadLine(filestream).toInt()
			instance.mExecIntervallVary = ReadLine(filestream).toInt()
			
			While Not Eof(filestream)
				Local instanceentry:TPatternEntry = New TPatternEntry
					instanceentry.mBlueprint = TVesselBlueprint.get(ReadLine(filestream))
					instanceentry.mX = ReadLine(filestream).toFloat()
					instanceentry.mY = ReadLine(filestream).toFloat()
				
				'Only add if we have a valid blueprint
				If(instanceentry.mBlueprint) Then instance.mPatternList.addLast(instanceentry)
			Wend
	Next
End Function

Function prepareGameEnvi()
	gPlayerHealth = PLAYERMAXHEALTH
	gPlayerWeaponBar = 0
	gPlayerScore = 0
	gPlayerLastShot = MilliSecs()
	gGameOver = False
	
	gNextBonusCast = MilliSecs()+BONUSCASTINTERVAL*Rnd(BONUSCASTMODIFIERMIN,BONUSCASTMODIFIERMAX)
	
	gPlayerX = GraphicsWidth()/2
	gPlayerY = GraphicsHeight()-100
	MoveMouse(gPlayerX,gPlayerY)
	
	For Local pattern:TPattern = EachIn TPattern.gList
		pattern.mNextExec = 0
	Next
	
	TProjectile.gList.Clear()
	TVessel.gList.Clear()
	TExplosion.gList.Clear()
	TBonus.gList.Clear()
End Function

Function saveHighscore()
	If gPlayerScore > gPlayerMaxScore
		gPlayerMaxScore = gPlayerScore
		
		Local filestream:TStream = WriteFile("score.txt")
		If(filestream)
			WriteInt(filestream,gPlayerMaxScore )
			CloseStream(filestream)
		EndIf
	EndIf
End Function

Function loadHighscore()
	If FileType("score.txt") = 1
		Local filestream:TStream = ReadFile("score.txt")
		
		If(filestream)
			gPlayerMaxScore = ReadInt(filestream)
			Local fullscreen:String = ReadLine(filestream).toLower()
			CloseStream(filestream)
		EndIf
	EndIf
End Function